@@ -32,7 +32,10 @@ class BinariesCheck(AbstractCheck):
     la_file_regex = re.compile(r'\.la$')
     invalid_dir_ref_regex = re.compile(r'/(home|tmp)(\W|$)')
     usr_arch_share_regex = re.compile(r'/share/.*/(?:x86|i.86|x86_64|ppc|ppc64|s390|s390x|ia64|m68k|arm|aarch64|mips|riscv)')
-    python_module_regex = re.compile(r'.*\.\w*(python|pypy)\w*(-\w+){4}\.so')
+    python_module_regex = re.compile(r'.*\.(\w*(python|pypy)\w*(-\w+){4}|abi3)\.so')
+    # Starts with ELF or with a special qualifiers like setuid or setgid
+    # https://github.com/rpm-software-management/rpmlint/issues/1088
+    elf_regex = re.compile(r'^(\w+ )?ELF ')
 
     lto_text_like_sections = {'.preinit_array', '.init_array', '.fini_array'}
     # The following sections are part of the RX ABI and do correspond to .text, .data and .bss
@@ -74,7 +77,8 @@ class BinariesCheck(AbstractCheck):
                                 self._check_forbidden_functions,
                                 self._check_executable_shlib,
                                 self._check_optflags,
-                                self._check_hash_sections]
+                                self._check_hash_sections,
+                                self._check_no_patchable_function_entries_in_archive]
 
     @staticmethod
     def create_nonlibc_regexp_call(call):
@@ -254,6 +258,18 @@ class BinariesCheck(AbstractCheck):
             self.output.add_info('E', pkg, 'lto-no-text-in-archive', pkgfile.name)
             return
 
+    def _check_no_patchable_function_entries_in_archive(self, pkg, pkgfile):
+        """
+        For static libraries, we should not ship __patchable_function_entries as it can
+        accidentally make a shared library or an executable live-patchable.
+        """
+        if self.is_archive:
+            for elf_file in self.readelf_parser.section_info.elf_files:
+                for section in elf_file:
+                    if section.name == '__patchable_function_entries':
+                        self.output.add_info('E', pkg, 'patchable-function-entry-in-archive', pkgfile.name)
+                        return
+
     def _check_missing_symtab_in_archive(self, pkg, pkgfile):
         """
         FIXME Add test coverage.
@@ -444,11 +460,8 @@ class BinariesCheck(AbstractCheck):
         dyn_section = self.readelf_parser.dynamic_section_info
         if not len(dyn_section.needed) and not (dyn_section.soname and
                                                 self.ldso_soname_regex.search(dyn_section.soname)):
-            if self.is_shobj:
-                msg = 'shared-library-without-dependency-information'
-            else:
-                msg = 'statically-linked-binary'
-            self.output.add_info('E', pkg, msg, pkgfile.name)
+            if not self.is_shobj:
+                self.output.add_info('E', pkg, 'statically-linked-binary', pkgfile.name)
         else:
             # linked against libc ?
             if 'libc.' not in dyn_section.runpaths and \
@@ -458,11 +471,8 @@ class BinariesCheck(AbstractCheck):
                 for lib in dyn_section.needed:
                     if 'libc.' in lib:
                         return
-                if self.is_shobj:
-                    msg = 'library-not-linked-against-libc'
-                else:
-                    msg = 'program-not-linked-against-libc'
-                self.output.add_info('W', pkg, msg, pkgfile.name)
+                if not self.is_shobj:
+                    self.output.add_info('W', pkg, 'program-not-linked-against-libc', pkgfile.name)
 
     def _check_forbidden_functions(self, pkg, pkgfile):
         forbidden_functions = self.config.configuration['WarnOnFunction']
@@ -499,7 +509,7 @@ class BinariesCheck(AbstractCheck):
                 continue
 
             # See: https://github.com/PyCQA/flake8-bugbear/issues/269
-            waiver = any(map(lambda string: f['waiver_regex'].search(string), strings_parser.strings))  # noqa: B023
+            waiver = any(f['waiver_regex'].search(string) for string in strings_parser.strings)
             if not waiver:
                 forbidden_functions_filtered.append(fn)
 
@@ -637,17 +647,18 @@ class BinariesCheck(AbstractCheck):
 
             # find out if we have a file in /usr/lib64/ directory (needed later
             # for the package checks)
-            if stat.S_ISREG(pkgfile.mode) and \
-                    (fname.startswith('/usr/lib64') or
-                     fname.startswith('/lib64')):
+            if fname.startswith(('/usr/lib64', '/lib64')):
                 pkg_has_file_in_lib64 = True
 
             # skip the rest of the tests for non-binaries
             # binary files only from here on
             is_ocaml_native = 'Objective caml native' in pkgfile.magic
             is_lua_bytecode = 'Lua bytecode' in pkgfile.magic
-            if not (pkgfile.magic.startswith('ELF ') or 'current ar archive'
-                    in pkgfile.magic or is_ocaml_native or is_lua_bytecode):
+            # Look for ELF in the file magic to check if it's really a binary
+            # file
+            if not (self.elf_regex.match(pkgfile.magic) or
+                    'current ar archive' in pkgfile.magic or
+                    is_ocaml_native or is_lua_bytecode):
                 continue
 
             self.checked_files += 1
