@@ -2,12 +2,15 @@ import bz2
 from collections import namedtuple
 import contextlib
 import gzip
+import hashlib
+import io
 import lzma
 import mmap
 import os
-from pathlib import Path
+from pathlib import Path, PurePath
 import re
 from shlex import quote
+import shutil
 import stat
 import subprocess
 import tempfile
@@ -16,12 +19,9 @@ from urllib.parse import urljoin
 
 try:
     import magic
-    # TODO: magic.MAGIC_COMPRESS when PkgFile gets decompress support.
-    _magic = magic.open(magic.MAGIC_NONE)
-    _ = _magic.descriptor  # magic >= 5.05 needed
-    _magic.load()
+    has_magic = True
 except ImportError:
-    _magic = None
+    has_magic = False
 import rpm
 from rpmlint.helpers import byte_to_string, ENGLISH_ENVIROMENT, print_warning
 from rpmlint.pkgfile import PkgFile
@@ -122,7 +122,7 @@ def is_utf8_bytestr(s):
         else:
             unexpected = type(s).__name__
             raise TypeError(
-                'Expected str/unicode/bytes, not {}'.format(unexpected))
+                f'Expected str/unicode/bytes, not {unexpected}')
     except UnicodeError:
         return False
     return True
@@ -247,7 +247,7 @@ def formatRequire(name, flags, evr):
             s = s + '>'
         if flags & rpm.RPMSENSE_EQUAL:
             s = s + '='
-        s = '%s %s' % (s, versionToString(evr))
+        s = f'{s} {versionToString(evr)}'
     return s
 
 
@@ -274,7 +274,7 @@ def stringToVersion(verstring):
     if i != -1:
         with contextlib.suppress(ValueError):
             # garbage in epoch, ignore it
-            epoch = str(int(verstring[:i]))
+            epoch = int(verstring[:i])
     i += 1
     j = verstring.find('-', i)
     if j != -1:
@@ -357,15 +357,25 @@ def parse_deps(line):
     return prcos
 
 
+def _get_magic_libmagic(path):
+    return magic.detect_from_filename(path).name
+
+
+def _get_magic_python_magic(path):
+    return magic.from_file(path)
+
+
 def get_magic(path):
-    # file() method evaluates every file twice with python2,
-    # use descriptor() method instead
+    # python-magic & libmagic compatibility code
+    # https://github.com/ahupp/python-magic/blob/master/COMPAT.md
+    detect_magic = _get_magic_python_magic
+    if not hasattr(magic, 'from_file'):
+        # libmagic python bindings
+        detect_magic = _get_magic_libmagic
+
     try:
-        fd = os.open(path, os.O_RDONLY)
-        magic = byte_to_string(_magic.descriptor(fd))
-        os.close(fd)
-        return magic
-    except OSError:
+        return detect_magic(path)
+    except (ValueError, FileNotFoundError):
         return ''
 
 
@@ -375,6 +385,87 @@ class AbstractPkg:
     def cleanup(self):
         pass
 
+    # internal function to gather dependency info used by the above ones
+    def _gather_aux(self, header, xs, nametag, flagstag, versiontag,
+                    prereq=None):
+        names = header[nametag]
+        flags = header[flagstag]
+        versions = header[versiontag]
+
+        if versions:
+            for loop in range(len(versions)):
+                name = byte_to_string(names[loop])
+                evr = stringToVersion(byte_to_string(versions[loop]))
+                if prereq is not None and flags[loop] & PREREQ_FLAG:
+                    prereq.append((name, flags[loop] & (~PREREQ_FLAG), evr))
+                else:
+                    xs.append(DepInfo(name, flags[loop], evr))
+        return xs, prereq
+
+    def _gather_dep_info(self):
+        _requires = []
+        _prereq = []
+        _provides = []
+        _conflicts = []
+        _obsoletes = []
+        _recommends = []
+        _suggests = []
+        _enhances = []
+        _supplements = []
+
+        _requires, _prereq = self._gather_aux(self.header, _requires,
+                                              rpm.RPMTAG_REQUIRENAME,
+                                              rpm.RPMTAG_REQUIREFLAGS,
+                                              rpm.RPMTAG_REQUIREVERSION,
+                                              _prereq)
+        _conflits, _ = self._gather_aux(self.header, _conflicts,
+                                        rpm.RPMTAG_CONFLICTNAME,
+                                        rpm.RPMTAG_CONFLICTFLAGS,
+                                        rpm.RPMTAG_CONFLICTVERSION)
+        _provides, _ = self._gather_aux(self.header, _provides,
+                                        rpm.RPMTAG_PROVIDENAME,
+                                        rpm.RPMTAG_PROVIDEFLAGS,
+                                        rpm.RPMTAG_PROVIDEVERSION)
+        _obsoletes, _ = self._gather_aux(self.header, _obsoletes,
+                                         rpm.RPMTAG_OBSOLETENAME,
+                                         rpm.RPMTAG_OBSOLETEFLAGS,
+                                         rpm.RPMTAG_OBSOLETEVERSION)
+        _recommends, _ = self._gather_aux(self.header, _recommends,
+                                          rpm.RPMTAG_RECOMMENDNAME,
+                                          rpm.RPMTAG_RECOMMENDFLAGS,
+                                          rpm.RPMTAG_RECOMMENDVERSION)
+        _suggests, _ = self._gather_aux(self.header, _suggests,
+                                        rpm.RPMTAG_SUGGESTNAME,
+                                        rpm.RPMTAG_SUGGESTFLAGS,
+                                        rpm.RPMTAG_SUGGESTVERSION)
+        _enhances, _ = self._gather_aux(self.header, _enhances,
+                                        rpm.RPMTAG_ENHANCENAME,
+                                        rpm.RPMTAG_ENHANCEFLAGS,
+                                        rpm.RPMTAG_ENHANCEVERSION)
+        _supplements, _ = self._gather_aux(self.header, _supplements,
+                                           rpm.RPMTAG_SUPPLEMENTNAME,
+                                           rpm.RPMTAG_SUPPLEMENTFLAGS,
+                                           rpm.RPMTAG_SUPPLEMENTVERSION)
+
+        return (_requires, _prereq, _provides, _conflicts, _obsoletes, _recommends,
+                _suggests, _enhances, _supplements)
+
+    def scriptprog(self, which):
+        """
+        Get the specified script interpreter as a string.
+        Depending on rpm-python version, the string may or may not include
+        interpreter arguments, if any.
+        """
+        if which is None:
+            return ''
+        prog = self[which]
+        if prog is None:
+            prog = ''
+        elif isinstance(prog, (list, tuple)):
+            # http://rpm.org/ticket/847#comment:2
+            prog = ''.join(prog)
+        return prog
+
     def __enter__(self):
         return self
 
@@ -392,7 +483,7 @@ class Pkg(AbstractPkg):
         # record decompression and extraction time
         start = time.monotonic()
         self.dirname = self._extract_rpm(dirname, verbose)
-        self.extraction_time = time.monotonic() - start
+        self.timers = {'rpm2cpio': time.monotonic() - start, 'libmagic': 0}
         self.current_linenum = None
 
         self._req_names = -1
@@ -497,8 +588,8 @@ class Pkg(AbstractPkg):
     def check_signature(self):
         ret = subprocess.run(('rpm', '-Kv', self.filename),
                              stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                             env=ENGLISH_ENVIROMENT)
-        text = ret.stdout.decode()
+                             env=ENGLISH_ENVIROMENT, text=True)
+        text = ret.stdout
         if text.endswith('\n'):
             text = text[:-1]
         return ret.returncode, text
@@ -590,8 +681,10 @@ class Pkg(AbstractPkg):
                     elif not pkgfile.size:
                         pkgfile.magic = 'empty'
                 if (not pkgfile.magic and
-                        not pkgfile.is_ghost and _magic):
+                        not pkgfile.is_ghost and has_magic):
+                    start = time.monotonic()
                     pkgfile.magic = get_magic(pkgfile.path)
+                    self.timers['libmagic'] += time.monotonic() - start
                 if pkgfile.magic is None or Pkg._magic_from_compressed_re.search(pkgfile.magic):
                     # Discard magic from inside compressed files ('file -z')
                     # until PkgFile gets decompression support.  We may get
@@ -627,87 +720,6 @@ class Pkg(AbstractPkg):
                 return True
         return False
 
-    # internal function to gather dependency info used by the above ones
-    def _gather_aux(self, header, xs, nametag, flagstag, versiontag,
-                    prereq=None):
-        names = header[nametag]
-        flags = header[flagstag]
-        versions = header[versiontag]
-
-        if versions:
-            for loop in range(len(versions)):
-                name = byte_to_string(names[loop])
-                evr = stringToVersion(byte_to_string(versions[loop]))
-                if prereq is not None and flags[loop] & PREREQ_FLAG:
-                    prereq.append((name, flags[loop] & (~PREREQ_FLAG), evr))
-                else:
-                    xs.append(DepInfo(name, flags[loop], evr))
-        return xs, prereq
-
-    def _gather_dep_info(self):
-        _requires = []
-        _prereq = []
-        _provides = []
-        _conflicts = []
-        _obsoletes = []
-        _recommends = []
-        _suggests = []
-        _enhances = []
-        _supplements = []
-
-        _requires, _prereq = self._gather_aux(self.header, _requires,
-                                              rpm.RPMTAG_REQUIRENAME,
-                                              rpm.RPMTAG_REQUIREFLAGS,
-                                              rpm.RPMTAG_REQUIREVERSION,
-                                              _prereq)
-        _conflits, _ = self._gather_aux(self.header, _conflicts,
-                                        rpm.RPMTAG_CONFLICTNAME,
-                                        rpm.RPMTAG_CONFLICTFLAGS,
-                                        rpm.RPMTAG_CONFLICTVERSION)
-        _provides, _ = self._gather_aux(self.header, _provides,
-                                        rpm.RPMTAG_PROVIDENAME,
-                                        rpm.RPMTAG_PROVIDEFLAGS,
-                                        rpm.RPMTAG_PROVIDEVERSION)
-        _obsoletes, _ = self._gather_aux(self.header, _obsoletes,
-                                         rpm.RPMTAG_OBSOLETENAME,
-                                         rpm.RPMTAG_OBSOLETEFLAGS,
-                                         rpm.RPMTAG_OBSOLETEVERSION)
-        _recommends, _ = self._gather_aux(self.header, _recommends,
-                                          rpm.RPMTAG_RECOMMENDNAME,
-                                          rpm.RPMTAG_RECOMMENDFLAGS,
-                                          rpm.RPMTAG_RECOMMENDVERSION)
-        _suggests, _ = self._gather_aux(self.header, _suggests,
-                                        rpm.RPMTAG_SUGGESTNAME,
-                                        rpm.RPMTAG_SUGGESTFLAGS,
-                                        rpm.RPMTAG_SUGGESTVERSION)
-        _enhances, _ = self._gather_aux(self.header, _enhances,
-                                        rpm.RPMTAG_ENHANCENAME,
-                                        rpm.RPMTAG_ENHANCEFLAGS,
-                                        rpm.RPMTAG_ENHANCEVERSION)
-        _supplements, _ = self._gather_aux(self.header, _supplements,
-                                           rpm.RPMTAG_SUPPLEMENTNAME,
-                                           rpm.RPMTAG_SUPPLEMENTFLAGS,
-                                           rpm.RPMTAG_SUPPLEMENTVERSION)
-
-        return (_requires, _prereq, _provides, _conflicts, _obsoletes, _recommends,
-                _suggests, _enhances, _supplements)
-
-    def scriptprog(self, which):
-        """
-        Get the specified script interpreter as a string.
-        Depending on rpm-python version, the string may or may not include
-        interpreter arguments, if any.
-        """
-        if which is None:
-            return ''
-        prog = self[which]
-        if prog is None:
-            prog = ''
-        elif isinstance(prog, (list, tuple)):
-            # http://rpm.org/ticket/847#comment:2
-            prog = ''.join(prog)
-        return prog
-
 
 def get_installed_pkgs(name):
     """Get list of installed package objects by name."""
@@ -754,6 +766,17 @@ class InstalledPkg(Pkg):
 
 # Class to provide an API to a 'fake' package, eg. for specfile-only checks
 class FakePkg(AbstractPkg):
+    _autoheaders = [
+        'requires',
+        'conflicts',
+        'provides',
+        'obsoletes',
+        'recommends',
+        'suggests',
+        'enhances',
+        'supplements',
+    ]
+
     def __init__(self, name, is_source=False):
         self.name = str(name)
         self.arch = None
@@ -765,26 +788,130 @@ class FakePkg(AbstractPkg):
         self.files = {}
         self.ghost_files = {}
 
+        # header is a dictionary to mock rpm metadata
+        self.header = {}
+        for i in self._autoheaders:
+            # the header name wihtout the ending 's'
+            tagname = i[:-1].upper()
+            self.header[getattr(rpm, f'RPMTAG_{tagname}NAME')] = []
+            self.header[getattr(rpm, f'RPMTAG_{tagname}FLAGS')] = []
+            self.header[getattr(rpm, f'RPMTAG_{tagname}VERSION')] = []
+        self.header[rpm.RPMTAG_FILENAMES] = []
+
     def add_file(self, path, name):
         pkgfile = PkgFile(name)
         pkgfile.path = path
         self.files[name] = pkgfile
         return pkgfile
 
-    def add_file_with_content(self, name, content, **flags):
+    def _mock_file(self, path, attrs):
+        metadata = None
+        if attrs.get('create_dirs', False):
+            for i in PurePath(path).parents[:attrs.get('include_dirs', -1)]:
+                self.add_dir(str(i))
+        metadata = attrs.get('metadata', None)
+
+        content = ''
+        if 'content-path' in attrs:
+            content = open(attrs['content-path'])
+        elif 'content' in attrs:
+            content = attrs['content']
+
+        if 'linkto' in attrs:
+            self.add_symlink_to(path, attrs['linkto'])
+        else:
+            self.add_file_with_content(path, content, metadata=metadata)
+        self.header[rpm.RPMTAG_FILENAMES].append(path)
+
+        if 'content-path' in attrs:
+            content.close()
+
+    def create_files(self, files):
+        """
+        This is a helper method to create files(real files); not PkgFile
+        objects.
+        """
+
+        # files can be just a list
+        if isinstance(files, list) or isinstance(files, tuple):
+            for path in files:
+                self._mock_file(path, {})
+        # list of files with attributes and content
+        elif isinstance(files, dict):
+            for path, file in files.items():
+                self._mock_file(path, file)
+
+    def add_dir(self, path):
+        pkgdir = PkgFile(path)
+        pkgdir.magic = 'directory'
+        pkgdir.path = path
+        self.files[path] = pkgdir
+        return pkgdir
+
+    def add_file_with_content(self, name, content, metadata=None, **flags):
         """
         Add file to the FakePkg and fill the file with provided
         string content.
         """
-        basename = name.replace(os.path.sep, '_')
-        path = os.path.join(self.dir_name(), basename)
-        with open(path, 'w') as out:
-            out.write(content)
-            pkg_file = PkgFile(name)
-            pkg_file.path = path
-            for key, value in flags.items():
-                setattr(pkg_file, key, value)
-            self.files[name] = pkg_file
+        path = os.path.join(self.dir_name(), name.lstrip('/'))
+        pkg_file = PkgFile(name)
+        pkg_file.path = path
+        pkg_file.mode = stat.S_IFREG | 0o0644
+        pkg_file.user = 'root'
+        pkg_file.group = 'root'
+        self.files[name] = pkg_file
+
+        # create files in filesystem
+        os.makedirs(Path(path).parent, exist_ok=True)
+
+        with open(Path(path), 'w') as out:
+            # file like content
+            if isinstance(content, io.IOBase):
+                shutil.copyfileobj(content, out)
+            # text content
+            else:
+                out.write(content)
+
+        # Generating md5 hash values for real files:
+        pkg_file.md5 = self.md5_checksum(Path(path))
+        pkg_file.size = os.path.getsize(Path(path))
+        pkg_file.inode = os.stat(Path(path)).st_ino
+
+        if metadata:
+            for k, v in metadata.items():
+                setattr(pkg_file, k, v)
+        for key, value in flags.items():
+            setattr(pkg_file, key, value)
+
+    def initiate_files_base_data(self):
+        """ This method is called after adding metadata of each file """
+        self.config_files = [x.name for x in self.files.values() if x.is_config]
+        self.doc_files = [x.name for x in self.files.values() if x.is_doc]
+        self.ghost_files = [x.name for x in self.files.values() if x.is_ghost]
+        self.noreplace_files = [x.name for x in self.files.values() if x.is_noreplace]
+        self.missingok_files = [x.name for x in self.files.values() if x.is_missingok]
+
+    def add_header(self, header):
+        for k, v in header.items():
+            if k in self._autoheaders:
+                # the header name wihtout the ending 's'
+                tagname = k[:-1].upper()
+                for i in v:
+                    name, flags, version = parse_deps(i)[0]
+                    version = f'{version[1]}-{version[2]}'
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}NAME')].append(name)
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}FLAGS')].append(flags)
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}VERSION')].append(version)
+                continue
+
+            key = getattr(rpm, f'RPMTAG_{k}'.upper())
+            self.header[key] = v
+
+        (self.requires, self.prereq, self.provides, self.conflicts,
+         self.obsoletes, self.recommends, self.suggests, self.enhances,
+         self.supplements) = self._gather_dep_info()
+
+        self.req_names = [x[0] for x in self.requires + self.prereq]
 
     def add_symlink_to(self, name, target):
         """
@@ -795,6 +922,8 @@ class FakePkg(AbstractPkg):
         pkg_file = PkgFile(name)
         pkg_file.mode = stat.S_IFLNK
         pkg_file.linkto = target
+        pkg_file.user = 'root'
+        pkg_file.group = 'root'
         self.files[name] = pkg_file
 
     def readlink(self, pkgfile):
@@ -807,6 +936,17 @@ class FakePkg(AbstractPkg):
             self.dirname = self.__tmpdir.name
         return self.dirname
 
+    def md5_checksum(self, file_name):
+        md5_hash = hashlib.md5()
+        with open(file_name, 'rb') as f:
+            for byte_block in iter(lambda: f.read(4096), b''):
+                md5_hash.update(byte_block)
+        return md5_hash.hexdigest()
+
     def cleanup(self):
         if self.dirname:
             self.__tmpdir.cleanup()
+
+    # access the tags like an array
+    def __getitem__(self, key):
+        return self.header.get(key, None)
