@@ -12,19 +12,36 @@ def speccheck():
     CONFIG.info = True
     output = Filter(CONFIG)
     test = SpecCheck(CONFIG, output)
-    return output, test
+    yield output, test
 
 
-def test_check_include(tmpdir, speccheck):
+@pytest.fixture
+def output(speccheck):
+    output, _test = speccheck
+    yield output
+
+
+@pytest.fixture
+def test(speccheck):
+    _output, test = speccheck
+    yield test
+
+
+def test_check_include(tmp_path, speccheck):
     output, test = speccheck
-    test.check_source(get_tested_package('source/CheckInclude', tmpdir))
+    test.check_source(get_tested_package('source/CheckInclude', tmp_path))
     out = output.print_results(output.results)
     assert "specfile-error can't parse specfile" not in out
     assert 'no-buildroot-tag' in out
     assert 'E: specfile-error error: query of specfile' not in out
 
 
-@pytest.mark.parametrize('package', ['spec/SpecCheck2', 'spec/SpecCheck3'])
+@pytest.mark.parametrize('package', [
+    'spec/SpecCheck2',
+    'spec/SpecCheck3',
+    'spec/SpecCheckPatch',
+    'spec/intltool',
+])
 def test_patch_not_applied(package, speccheck):
     output, test = speccheck
     pkg = get_tested_spec_package(package)
@@ -67,19 +84,19 @@ def test_forbidden_controlchars_found(package, speccheck):
 
 
 @pytest.mark.parametrize('package', ['source/no-spec-file'])
-def test_check_no_spec_file(tmpdir, package, speccheck):
+def test_check_no_spec_file(tmp_path, package, speccheck):
     """Test if spec file is not found inside RPM metadata."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: no-spec-file' in out
 
 
 @pytest.mark.parametrize('package', ['source/CheckInclude'])
-def test_check_no_spec_file_not_applied(tmpdir, package, speccheck):
+def test_check_no_spec_file_not_applied(tmp_path, package, speccheck):
     """Test if there is no spec file inside RPM metadata."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: no-spec-file' not in out
 
@@ -105,19 +122,19 @@ def test_check_non_utf8_spec_file_not_applied(package, speccheck):
 
 
 @pytest.mark.parametrize('package', ['source/invalid-spec-name'])
-def test_check_invalid_spec_name(tmpdir, package, speccheck):
+def test_check_invalid_spec_name(tmp_path, package, speccheck):
     """Test if specfile name does not matches the ('Name: ') tag."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: invalid-spec-name' in out
 
 
 @pytest.mark.parametrize('package', ['source/CheckInclude'])
-def test_check_invalid_spec_name_not_applied(tmpdir, package, speccheck):
+def test_check_invalid_spec_name_not_applied(tmp_path, package, speccheck):
     """Test if specfile has specfile name as ('Name: ') tag."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: invalid-spec-name' not in out
 
@@ -1102,3 +1119,75 @@ def test_python_setup_test(package, speccheck):
     test.check_spec(pkg)
     out = output.print_results(output.results)
     assert 'W: python-setup-test' in out
+
+
+@pytest.mark.parametrize('package', ['spec/python-module-def'])
+def test_python_module_definition(package, speccheck):
+    """Test if python_module macro is defined in the spec file."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-module-def' in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/SpecCheck',
+    'spec/SpecCheck2',
+    'spec/SpecCheck3',
+    'spec/SpecCheck4',
+])
+def test_python_module_definition_not_present(package, speccheck):
+    """Test if python_module macro warning is not shown if the macro is not
+       defined in the spec file.
+    """
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-module-def' not in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/python-sitelib-glob',
+    'spec/python-sitearch-glob',
+])
+def test_python_sitelib_glob(package, speccheck):
+    """Test if %{python_sitelib}/* is present in %files section."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-sitelib-glob-in-files' in out
+
+
+@pytest.mark.parametrize('package', ['spec/python-sitelib'])
+def test_python_sitelib(package, speccheck):
+    """Test if %{python_sitelib}/* is present in %files section."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-sitelib-glob-in-files' not in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/null-char-last',
+    'spec/null-char-first',
+])
+def test_null_char(package, speccheck):
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'forbidden-controlchar-found' in out
+
+
+@pytest.mark.parametrize('package', [
+    get_tested_spec_package('spec/MacroInComment'),
+])
+def test_special_comments(package, output, test):
+    test.check_spec(package)
+    out = output.print_results(output.results)
+    assert 'W: macro-in-comment %configure' in out
+    assert 'W: macro-in-comment %{name}' not in out
